package AccessCheck::App::Controller;

use Mojo::Base qw(Mojolicious::Controller);

use English qw(-no_match_vars);
use HTTP::AcceptLanguage;
use Syntax::Keyword::Try;

use AccessCheck::Data::DB;
use AccessCheck::Data::Entity;
use AccessCheck::Data::Token;
use AccessCheck::L10N;
use AccessCheck::Regexp;

sub init_l10n {
    my $self = shift;

    my $log = $self->app()->log();

    # lang identification first, as needed for any further error message
    my ($l10n, $lang);
    if ($lang = $self->param('lang')) {
        $log->debug(sprintf("setting language from parameter: %s", $lang));
    } elsif ($lang = $self->session('lang')) {
        $log->debug(sprintf("setting language from session: %s", $lang));
    } elsif (my $header = $self->req()->headers->header('Accept-Language')) {
        $lang = HTTP::AcceptLanguage->new($header)->match(qw/en fr/);
        $log->debug(sprintf("setting language from Accept-Language header: %s", $lang));
    } else {
        $lang = 'en';
    }

    $l10n = AccessCheck::L10N->get_handle($lang);

    $self->session(lang => $lang);
    $self->stash(lang => $lang);
    $self->stash(l10n => $l10n);

    return $l10n;
}

sub init_user {
    my $self = shift;

    my $headers = $self->req()->headers();

    my $idp = 
        $ENV{'Shib_Identity_Provider'} ||           # local SP
        $headers->header('Shib-Identity-Provider'); # remote SP

    my $name =
        $ENV{displayName} ||             # local SP
        $headers->header('displayName'); # remote SP

    my $user = {
        idp => $idp,
        name => $name
    };

    $self->stash(user => $user);

    return $user;
}

sub init_db {
    my $self = shift;

    my $db = AccessCheck::Data::DB->new();

    $self->stash(db => $db);

    return $db;
}

sub check_authentication {
    my $self = shift;

    return $self->abort(
        status       => 401,
        log_message  => sprintf("unauthenticated user for action %s",  $self->current_route()),
        user_message => Registry::Error::AuthenticationRequired->new()
    ) if !$self->stash('user');

    return 1;
}

sub check_token {
    my ($self, %args) = @_;

    my $secret = $args{token};
    my $db     = $self->stash('db');

    my $token = AccessCheck::Data::Token->new(
        db     => $db,
        secret => $secret
    );

    return $self->abort(
        status       => 400,
        log_message  => "No such authentication token $secret",
        user_message => "wrong_token"
    ) if !$token->load(speculative => 1);

    return $self->abort(
        status       => 400,
        log_message  => "Authentication token $secret cannot be used for SP $args{entityid}",
        user_message => "wrong_token_for_sp"
    ) if $token->entityid() ne $args{entityid};

    ## delete the token
    try {
        $token->delete();
    } catch {
        $self->app()->log()->error(
            sprintf("Failed to delete authentication token %s", $secret)
        );
    }

    return 1;
}

=head2 check_csrf_token()

Check if provided anti-CSRF token, as I<token> parameter, matches expected
one, abort otherwise.

=cut

sub check_csrf_token {
    my ($self, %args) = @_;

    my $provided_token = $self->param('token');
    return $self->abort(
        status       => 403,
        log_message  => sprintf("missing anti-CSRF token for action %s", $self->current_route()),
        user_message => "missing_csrf_token"
    ) if !$provided_token;

    my $expected_token = $self->csrf_token();

    return $self->abort(
        status      => 403,
        log_message => sprintf(
            "invalid anti-CSRF token for action %s: %s instead of expected %s",
            $self->current_route(),
            $provided_token,
            $expected_token,
        ),
        user_message => "invalid_csrf_token"
    ) if $provided_token ne $expected_token;

    return 1;
}

sub get_sp {
    my ($self, %args) = @_;

    my $entityid = $args{entityid};
    my $db       = $self->stash('db');

    return $self->abort(
        log_message  => "Missing parameter: entityid",
        user_message => "missing_entityid"
    ) if !$entityid;

    return $self->abort(
        log_message  => "Invalid parameter: entityid",
        user_message => "invalid_entityid"
    ) if $entityid !~ $AccessCheck::Regexp::entityid;


    my $sp = AccessCheck::Data::Entity->new(
        db       => $db,
        entityid => $entityid
    );

    return $self->abort(
        log_message  => sprintf("No such SP '%s' in database", $entityid),
        user_message => "no_such_entity"
    ) if !$sp->load(speculative => 1);

    return $sp;
}

sub abort {
    my $self = shift;
    my %args = @_;

    my $status = $args{status} || 200;
    my $format = $args{format} || 'html';
    my $db     = $self->stash('db');

    $db->rollback() if $db && $db->in_transaction();

    $self->app()->log()->error($args{log_message}) if $args{log_message};

    $self->stash(error => $args{user_message});
    $self->render(status => $status, template => 'errors', format => 'html');

    return;
}

sub loc {
    my $self = shift;

    return $self->stash('l10n')->maketext(@_);
}

sub mock_contacts {
    my $self = shift;
    my $sp = shift;

    my $config    = $self->app()->config();
    my $entityid = $sp->entityid();

    my $contacts =
        $config->{$entityid}->{contacts} ||
        $config->{service}->{contacts};

    if ($contacts) {
        if ($contacts =~ /^\+(.+)/) {
            # complement original contacts
            $sp->contacts($sp->contacts(), $self->string_to_list($1));
        } else {
            # replace original contacts
            $sp->contacts($self->string_to_list($contacts));
        }
    }
}

1;
